@@ -1,5 +1,3 @@
-package AppConfig::File;
-
 #============================================================================
 #
 # AppConfig::File.pm
@@ -9,28 +7,17 @@ package AppConfig::File;
 #
 # Written by Andy Wardley <abw@wardley.org>
 #
-# Copyright (C) 1997-2003 Andy Wardley.  All Rights Reserved.
+# Copyright (C) 1997-2007 Andy Wardley.  All Rights Reserved.
 # Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
 #
-# $Id: File.pm,v 1.62 2004/02/04 10:28:28 abw Exp $
-#
 #============================================================================
 
+package AppConfig::File;
 use strict;
-
-require 5.005;
-
+use warnings;
 use AppConfig;
 use AppConfig::State;
-use File::HomeDir;
-
-use vars qw( $VERSION );
-BEGIN {
-	$VERSION = '1.64';
-}
-
-
-
+our $VERSION = '1.65';
 
 
 #------------------------------------------------------------------------
@@ -47,8 +34,7 @@ BEGIN {
 sub new {
     my $class = shift;
     my $state = shift;
-    
-    my $self = {
+    my $self  = {
         STATE    => $state,                # AppConfig::State ref
         DEBUG    => $state->_debug(),      # store local copy of debug 
         PEDANTIC => $state->_pedantic,     # and pedantic flags
@@ -56,9 +42,6 @@ sub new {
 
     bless $self, $class;
 
-    # Find the home directory
-    $self->{HOME} = File::HomeDir->my_home;
-
     # call parse(@_) to parse any files specified as further params
     $self->parse(@_) if @_;
 
@@ -107,67 +90,67 @@ sub parse {
 
     # ...and if it doesn't exist, we craft a default handler
     $errhandler = sub { warn(sprintf(shift, @_), "\n") }
-	unless defined $errhandler;
+        unless defined $errhandler;
 
     # install a closure as a new error handler
     $state->_ehandler(
-	sub {
-	    # modify the error message 
-	    my $format  = shift;
-	       $format .= ref $file 
-	                  ? " at line $."
-	                  : " at $file line $.";
+        sub {
+            # modify the error message 
+            my $format  = shift;
+               $format .= ref $file 
+                          ? " at line $."
+                          : " at $file line $.";
 
-	    # chain call to prevous handler
-	    &$errhandler($format, @_);
-	}
+            # chain call to prevous handler
+            &$errhandler($format, @_);
+        }
     );
 
     # trawl through all files passed as params
     FILE: while ($file = shift) {
 
-	# local/lexical vars ensure opened files get closed
-	my $handle;
-	local *FH;
+        # local/lexical vars ensure opened files get closed
+        my $handle;
+        local *FH;
 
-	# if the file is a reference, we assume it's a file handle, if
-	# not, we assume it's a filename and attempt to open it
-	$handle = $file;
-	if (ref($file)) {
-	    $handle = $file;
+        # if the file is a reference, we assume it's a file handle, if
+        # not, we assume it's a filename and attempt to open it
+        $handle = $file;
+        if (ref($file)) {
+            $handle = $file;
 
-	    # DEBUG
-	    print STDERR "reading from file handle: $file\n" if $debug;
-	}
-	else {
-	    # open and read config file
-	    open(FH, $file) or do {
-		# restore original error handler and report error
-	    	$state->_ehandler($errhandler);
-		$state->_error("$file: $!");
+            # DEBUG
+            print STDERR "reading from file handle: $file\n" if $debug;
+        }
+        else {
+            # open and read config file
+            open(FH, $file) or do {
+                # restore original error handler and report error
+                $state->_ehandler($errhandler);
+                $state->_error("$file: $!");
 
-		return undef;
-	    };
-	    $handle = \*FH;
+                return undef;
+            };
+            $handle = \*FH;
 
-	    # DEBUG
-	    print STDERR "reading file: $file\n" if $debug;
-	}
+            # DEBUG
+            print STDERR "reading file: $file\n" if $debug;
+        }
 
-	# initialise $prefix to nothing (no [block])
-	$prefix = '';
+        # initialise $prefix to nothing (no [block])
+        $prefix = '';
 
-	while (<$handle>) {
-	    chomp;
+        while (<$handle>) {
+            chomp;
 
             # Throw away everything from an unescaped # to EOL
             s/(^|\s+)#.*/$1/;
 
-	    # add next line if there is one and this is a continuation
-	    if (s/\\$// && !eof($handle)) {
-		$_ .= <$handle>;
-		redo;
-	    }
+            # add next line if there is one and this is a continuation
+            if (s/\\$// && !eof($handle)) {
+                $_ .= <$handle>;
+                redo;
+            }
 
             # Convert \# -> #
             s/\\#/#/g;
@@ -175,117 +158,117 @@ sub parse {
             # ignore blank lines
             next if /^\s*$/;
 
-	    # strip leading and trailing whitespace
-	    s/^\s+//;
-	    s/\s+$//;
+            # strip leading and trailing whitespace
+            s/^\s+//;
+            s/\s+$//;
 
-	    # look for a [block] to set $prefix
-	    if (/^\[([^\]]+)\]$/) {
-		$prefix = $1;
-		print STDERR "Entering [$prefix] block\n" if $debug;
-		next;
-	    }
+            # look for a [block] to set $prefix
+            if (/^\[([^\]]+)\]$/) {
+                $prefix = $1;
+                print STDERR "Entering [$prefix] block\n" if $debug;
+                next;
+            }
 
-	    # split line up by whitespace (\s+) or "equals" (\s*=\s*)
-	    if (/^([^\s=]+)(?:(?:(?:\s*=\s*)|\s+)(.*))?/) {
-		my ($variable, $value) = ($1, $2);
+            # split line up by whitespace (\s+) or "equals" (\s*=\s*)
+            if (/^([^\s=]+)(?:(?:(?:\s*=\s*)|\s+)(.*))?/) {
+                my ($variable, $value) = ($1, $2);
 
-		if (defined $value) {
-		    # here document
-		    if ($value =~ /^([^\s=]+\s*=)?\s*<<(['"]?)(\S+)\2$/) { # '<<XX' or 'hashkey =<<XX'
-			my $boundary = "$3\n";
-			$value = defined($1) ? $1 : '';
-			while (<$handle>) {
-			    last if $_ eq $boundary;
-			    $value .= $_;
-			};
-			$value =~ s/[\r\n]$//;
-		    } else {
-			# strip any quoting from the variable value
-			$value =~ s/^(['"])(.*)\1$/$2/;
-		    };
-		};
+                if (defined $value) {
+                    # here document
+                    if ($value =~ /^([^\s=]+\s*=)?\s*<<(['"]?)(\S+)\2$/) { # '<<XX' or 'hashkey =<<XX'
+                        my $boundary = "$3\n";
+                        $value = defined($1) ? $1 : '';
+                        while (<$handle>) {
+                            last if $_ eq $boundary;
+                            $value .= $_;
+                        };
+                        $value =~ s/[\r\n]$//;
+                    } else {
+                        # strip any quoting from the variable value
+                        $value =~ s/^(['"])(.*)\1$/$2/;
+                    };
+                };
 
-		# strip any leading '+/-' from the variable
-		$variable =~ s/^([\-+]?)//;
-		$flag = $1;
+                # strip any leading '+/-' from the variable
+                $variable =~ s/^([\-+]?)//;
+                $flag = $1;
 
-		# $variable gets any $prefix 
-		$variable = $prefix . '_' . $variable
-		    if length $prefix;
+                # $variable gets any $prefix 
+                $variable = $prefix . '_' . $variable
+                    if length $prefix;
 
-		# if the variable doesn't exist, we call set() to give 
-		# AppConfig::State a chance to auto-create it
-		unless ($state->_exists($variable) 
-			    || $state->set($variable, 1)) {
-		    $warnings++;
-		    last FILE if $pedantic;
-		    next;
-		}	
+                # if the variable doesn't exist, we call set() to give 
+                # AppConfig::State a chance to auto-create it
+                unless ($state->_exists($variable) 
+                            || $state->set($variable, 1)) {
+                    $warnings++;
+                    last FILE if $pedantic;
+                    next;
+                }       
 
-		my $nargs = $state->_argcount($variable);
+                my $nargs = $state->_argcount($variable);
 
-		# variables prefixed '-' are reset to their default values
-		if ($flag eq '-') {
-		    $state->_default($variable);
-		    next;
-		}
-		# those prefixed '+' get set to 1
-		elsif ($flag eq '+') {
-		    $value = 1 unless defined $value;
-		}
+                # variables prefixed '-' are reset to their default values
+                if ($flag eq '-') {
+                    $state->_default($variable);
+                    next;
+                }
+                # those prefixed '+' get set to 1
+                elsif ($flag eq '+') {
+                    $value = 1 unless defined $value;
+                }
 
-		# determine if any extra arguments were expected
-		if ($nargs) {
-		    if (defined $value && length $value) {
-			# expand any embedded variables, ~uids or
-			# environment variables, testing the return value
-			# for errors;  we pass in any variable-specific
-			# EXPAND value 
-			unless ($self->_expand(\$value, 
-				$state->_expand($variable), $prefix)) {
-			    print STDERR "expansion of [$value] failed\n" 
-				if $debug;
-			    $warnings++;
-			    last FILE if $pedantic;
-			}
-		    }
-		    else {
-			$state->_error("$variable expects an argument");
-			$warnings++;
-			last FILE if $pedantic;
-			next;
-		    }
-		}
-		# $nargs = 0
-		else {
-		    # default value to 1 unless it is explicitly defined
-		    # as '0' or "off"
-		    if (defined $value) {
-			# "off" => 0
-    			$value = 0 if $value =~ /off/i;
-			# any value => 1
-			$value = 1 if $value;
-		    }
-		    else {
-			# assume 1 unless explicitly defined off/0
-    			$value = 1;
-		    }
-		    print STDERR "$variable => $value (no expansion)\n"
-			if $debug;
-		}
-	   
-		# set the variable, noting any failure from set()
-		unless ($state->set($variable, $value)) {
-		    $warnings++;
-		    last FILE if $pedantic;
-		}
-	    }
-	    else {
-		$state->_error("parse error");
-		$warnings++;
-	    }
-	}
+                # determine if any extra arguments were expected
+                if ($nargs) {
+                    if (defined $value && length $value) {
+                        # expand any embedded variables, ~uids or
+                        # environment variables, testing the return value
+                        # for errors;  we pass in any variable-specific
+                        # EXPAND value 
+                        unless ($self->_expand(\$value, 
+                                $state->_expand($variable), $prefix)) {
+                            print STDERR "expansion of [$value] failed\n" 
+                                if $debug;
+                            $warnings++;
+                            last FILE if $pedantic;
+                        }
+                    }
+                    else {
+                        $state->_error("$variable expects an argument");
+                        $warnings++;
+                        last FILE if $pedantic;
+                        next;
+                    }
+                }
+                # $nargs = 0
+                else {
+                    # default value to 1 unless it is explicitly defined
+                    # as '0' or "off"
+                    if (defined $value) {
+                        # "off" => 0
+                        $value = 0 if $value =~ /off/i;
+                        # any value => 1
+                        $value = 1 if $value;
+                    }
+                    else {
+                        # assume 1 unless explicitly defined off/0
+                        $value = 1;
+                    }
+                    print STDERR "$variable => $value (no expansion)\n"
+                        if $debug;
+                }
+           
+                # set the variable, noting any failure from set()
+                unless ($state->set($variable, $value)) {
+                    $warnings++;
+                    last FILE if $pedantic;
+                }
+            }
+            else {
+                $state->_error("parse error");
+                $warnings++;
+            }
+        }
     }
 
     # restore original error handler
@@ -340,8 +323,8 @@ sub _expand {
     # create an AppConfig::Sys instance, or re-use a previous one, 
     # to handle platform dependant functions: getpwnam(), getpwuid()
     unless ($sys = $self->{ SYS }) {
-	require AppConfig::Sys;
-	$sys = $self->{ SYS } = AppConfig::Sys->new();
+        require AppConfig::Sys;
+        $sys = $self->{ SYS } = AppConfig::Sys->new();
     }
 
     print STDERR "Expansion of [$$value] " if $debug;
@@ -349,130 +332,129 @@ sub _expand {
     EXPAND: {
 
         # 
-	# EXPAND_VAR
-	# expand $(var) and $var as AppConfig::State variables
-	#
-	if ($expand & AppConfig::EXPAND_VAR) {
+        # EXPAND_VAR
+        # expand $(var) and $var as AppConfig::State variables
+        #
+        if ($expand & AppConfig::EXPAND_VAR) {
 
-	    $$value =~ s{
+            $$value =~ s{
                 (?<!\\)\$ (?: \((\w+)\) | (\w+) ) # $2 => $(var) | $3 => $var
 
-	    } {
-		# embedded variable name will be one of $2 or $3
-		$var = defined $1 ? $1 : $2;
+            } {
+                # embedded variable name will be one of $2 or $3
+                $var = defined $1 ? $1 : $2;
 
-		# expand the variable if defined
-		if ($state->_exists($var)) {
-		    $val = $state->get($var);
-		}
-		elsif (length $prefix 
-			&& $state->_exists($prefix . '_' . $var)) {
-		    print STDERR "(\$$var => \$${prefix}_$var) "
-			if $debug;
-		    $var = $prefix . '_' . $var;
-		    $val = $state->get($var);
-		}
-		else {
-		    # raise a warning if EXPAND_WARN set
-		    if ($expand & AppConfig::EXPAND_WARN) {
-			$state->_error("$var: no such variable");
-			$warnings++;
-		    }
+                # expand the variable if defined
+                if ($state->_exists($var)) {
+                    $val = $state->get($var);
+                }
+                elsif (length $prefix 
+                        && $state->_exists($prefix . '_' . $var)) {
+                    print STDERR "(\$$var => \$${prefix}_$var) "
+                        if $debug;
+                    $var = $prefix . '_' . $var;
+                    $val = $state->get($var);
+                }
+                else {
+                    # raise a warning if EXPAND_WARN set
+                    if ($expand & AppConfig::EXPAND_WARN) {
+                        $state->_error("$var: no such variable");
+                        $warnings++;
+                    }
 
-		    # replace variable with nothing
-		    $val = '';
-		}
+                    # replace variable with nothing
+                    $val = '';
+                }
 
-		# $val gets substituted back into the $value string
-		$val;
-	    }gex;
+                # $val gets substituted back into the $value string
+                $val;
+            }gex;
 
             $$value =~ s/\\\$/\$/g;
 
-	    # bail out now if we need to
-	    last EXPAND if $warnings && $pedantic;
-	}
+            # bail out now if we need to
+            last EXPAND if $warnings && $pedantic;
+        }
 
 
-	#
-	# EXPAND_UID
-	# expand ~uid as home directory (for $< if uid not specified)
-	#
-	if ($expand & AppConfig::EXPAND_UID) {
+        #
+        # EXPAND_UID
+        # expand ~uid as home directory (for $< if uid not specified)
+        #
+        if ($expand & AppConfig::EXPAND_UID) {
+            $$value =~ s{
+                ~(\w+)?                    # $1 => username (optional)
+            } {
+                $val = undef;
 
-	    $$value =~ s{
-		~(\w+)?                    # $1 => username (optional)
-	    } {
-		$val = undef;
+                # embedded user name may be in $1
+                if (defined ($var = $1)) {
+                    # try and get user's home directory
+                    if ($sys->can_getpwnam()) {
+                        $val = ($sys->getpwnam($var))[7];
+                    }
+                } else {
+                    # determine home directory 
+                    $val = $ENV{ HOME };
+                }
 
-		# embedded user name may be in $1
-		if (defined ($var = $1)) {
-		    # try and get user's home directory
-		    if ($sys->can_getpwnam()) {
-			$val = ($sys->getpwnam($var))[7];
-		    }
-		} else {
-		    # determine home directory 
-		    $val = $self->{ HOME };
-		}
+                # catch-all for undefined $dir
+                unless (defined $val) {
+                    # raise a warning if EXPAND_WARN set
+                    if ($expand & AppConfig::EXPAND_WARN) {
+                        $state->_error("cannot determine home directory%s",
+                            defined $var ? " for $var" : "");
+                        $warnings++;
+                    }
 
-		# catch-all for undefined $dir
-		unless (defined $val) {
-		    # raise a warning if EXPAND_WARN set
-		    if ($expand & AppConfig::EXPAND_WARN) {
-			$state->_error("cannot determine home directory%s",
-			    defined $var ? " for $var" : "");
-			$warnings++;
-		    }
+                    # replace variable with nothing
+                    $val = '';
+                }
 
-		    # replace variable with nothing
-		    $val = '';
-		}
+                # $val gets substituted back into the $value string
+                $val;
+            }gex;
 
-		# $val gets substituted back into the $value string
-		$val;
-	    }gex;
-
-	    # bail out now if we need to
-	    last EXPAND if $warnings && $pedantic;
-	}
+            # bail out now if we need to
+            last EXPAND if $warnings && $pedantic;
+        }
 
 
-	#
-	# EXPAND_ENV
-	# expand ${VAR} as environment variables
-	#
-	if ($expand & AppConfig::EXPAND_ENV) {
+        #
+        # EXPAND_ENV
+        # expand ${VAR} as environment variables
+        #
+        if ($expand & AppConfig::EXPAND_ENV) {
 
-	    $$value =~ s{ 
-		( \$ \{ (\w+) \} )
-	    } {
-		$var = $2;
+            $$value =~ s{ 
+                ( \$ \{ (\w+) \} )
+            } {
+                $var = $2;
 
-		# expand the variable if defined
-		if (exists $ENV{ $var }) {
-		    $val = $ENV{ $var };
-		} elsif ( $var eq 'HOME' ) {
-		    # In the special case of HOME, if not set
-		    # use the internal version
-		    $val = $self->{ HOME };
-		} else {
-		    # raise a warning if EXPAND_WARN set
-		    if ($expand & AppConfig::EXPAND_WARN) {
-			$state->_error("$var: no such environment variable");
-			$warnings++;
-		    }
+                # expand the variable if defined
+                if (exists $ENV{ $var }) {
+                    $val = $ENV{ $var };
+                } elsif ( $var eq 'HOME' ) {
+                    # In the special case of HOME, if not set
+                    # use the internal version
+                    $val = $self->{ HOME };
+                } else {
+                    # raise a warning if EXPAND_WARN set
+                    if ($expand & AppConfig::EXPAND_WARN) {
+                        $state->_error("$var: no such environment variable");
+                        $warnings++;
+                    }
 
-		    # replace variable with nothing
-		    $val = '';
-		}
-		# $val gets substituted back into the $value string
-		$val;
-	    }gex;
+                    # replace variable with nothing
+                    $val = '';
+                }
+                # $val gets substituted back into the $value string
+                $val;
+            }gex;
 
-	    # bail out now if we need to
-	    last EXPAND if $warnings && $pedantic;
-	}
+            # bail out now if we need to
+            last EXPAND if $warnings && $pedantic;
+        }
     }
 
     print STDERR "=> [$$value] (EXPAND = $expand)\n" if $debug;
@@ -493,9 +475,9 @@ sub _dump {
     my $self = shift;
 
     foreach my $key (keys %$self) {
-	printf("%-10s => %s\n", $key, 
-		defined($self->{ $key }) ? $self->{ $key } : "<undef>");
-    }	    
+        printf("%-10s => %s\n", $key, 
+                defined($self->{ $key }) ? $self->{ $key } : "<undef>");
+    }       
 } 
 
 
@@ -593,8 +575,8 @@ line with a '\'.
 
     callsign = alpha bravo camel delta echo foxtrot golf hipowls \
                india juliet kilo llama mike november oscar papa  \
-	       quebec romeo sierra tango umbrella victor whiskey \
-	       x-ray yankee zebra
+               quebec romeo sierra tango umbrella victor whiskey \
+               x-ray yankee zebra
 
 Variables that are simple flags and do not expect an argument (ARGCOUNT = 
 ARGCOUNT_NONE) can be specified without any value.  They will be set with 
@@ -644,7 +626,7 @@ A reference to the hash is returned when the variable is requested.
 
     my $aliases = $config->alias();
     foreach my $k (keys %$aliases) {
-	print "$k => $aliases->{ $k }\n";
+        print "$k => $aliases->{ $k }\n";
     }
 
 A large chunk of text can be defined using Perl's "heredoc" quoting
@@ -712,15 +694,9 @@ file.
 
 Andy Wardley, E<lt>abw@wardley.orgE<gt>
 
-=head1 REVISION
-
-$Revision: 1.62 $
-
 =head1 COPYRIGHT
 
-Copyright (C) 1997-2003 Andy Wardley.  All Rights Reserved.
-
-Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
+Copyright (C) 1997-2007 Andy Wardley.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or modify it 
 under the same terms as Perl itself.
